@@ -6,14 +6,15 @@ sss <- function(n, k, g) {
 #---- Variance matrix ----
 rs_var <- function(u, Z, X = Z, ids = seq_len(nrow(X)), df = NULL) {
   ids <- as.factor(ids)
-  if (is.null(df)) df <- sss(nrow(X), ncol(X), nlevels(ids))
+  df <- if (is.null(df)) {
+    sss(nrow(X), ncol(X), nlevels(ids))
+  } else {
+    as.numeric(df)
+  }
   # the meat
   ug <- split.data.frame(u, ids)
   Zg <- split.data.frame(Z, ids)
-  V <- lapply(seq_along(ug), 
-              function(i) {
-                tcrossprod(crossprod(Zg[[i]], ug[[i]]))
-              })
+  V <- Map(function(x, y) tcrossprod(crossprod(x, y)), Zg, ug)
   V <- Reduce(`+`, V)
   # the bread
   B <- solve(crossprod(Z, X))
